from sisense.resource import Resource
from sisense.api import API


class Permission(Resource):

    def __init__(self, api: API, rjson: dict = None, elasticube_name: str = None):
        super().__init__(api, rjson)
        self._elasticube = elasticube_name

    def all(self, elasticube: str = None) -> list:
        """
        Get permissions (shares) for the specified elasticube.

        :param elasticube: (str, default None) Elasticube's name. If None, use self.elasticube.
        :return: a list of permission objects
        """
        elasticube = elasticube if elasticube else self._elasticube
        content = self._api.get(f'elasticubes/localhost/{elasticube}/permissions')

        if 'shares' in content:
            permissions = [Permission(self._api, share, elasticube) for share in content['shares']]
        else:
            permissions = []

        return list(permissions)

    def get(self, oid: str, elasticube: str = None) -> Resource:
        """
        Get the permission for a user/group.

        :param oid: (str) Party ID.
        :param elasticube: (str, default None) Elasticube's name. If None, use self.elasticube.
        :return: (Permission) if found. None, otherwise.
        """
        for share in self.all(elasticube):
            if share.party == oid:
                return share

        return None

    def create(self, oid: str = None, ptype: str = None, level: str = None, elasticube: str = None) -> Resource:
        """
        Create new permission to the specified elasticube.
        If any optional parameters are None, add the current permission.

        :param oid: (str, default None) Group/user id.
        :param ptype: (str, default None) Party's type. Possible values are: 'group' or 'user'.
        :param level: (str, default None) Permission's type. Possible values are: 'r' (read) or 'w' (write).
        :param elasticube: (str, default None) Elasticube's name. If None, use self.elasticube.

        :return: (Permission) The new permission.
        """
        elasticube = elasticube if elasticube else self._elasticube
        data = self.json if oid is None or type is None or level is None else {'party': oid,
                                                                               'type': ptype,
                                                                               'permission': level}

        permissions = [share.json for share in self.all(elasticube)] + [data]
        self._api.put(f'elasticubes/localhost/{elasticube}/permissions', data=permissions)

        return Permission(self._api, data, elasticube)

    def delete(self):
        """Delete the current permission."""
        permissions = self.all()
        self._api.delete(f'elasticubes/localhost/{self._elasticube}/permissions')
        [share.create(share.party, share.type, share.permission) for share in permissions if share.party != self.party]

    def delete_all(self, elasticube: str = None):
        """
        Delete all elasticube's permissions.

        :param elasticube: (str, default None) Elasticube's name. If None, use self.elasticube.
        """
        elasticube = elasticube if elasticube else self._elasticube
        self._api.delete(f'elasticubes/localhost/{elasticube}/permissions')



from .test_case import TestCaseV1 as TestCase
from sisense.admin import User


class UserTestCase(TestCase):

    def test_get(self):
        user = self.sisense.user.get(self.config['user_email'])
        self.assertEqual(user.email, self.config['user_email'])

        same = self.sisense.user.get(oid=user._id)
        self.assertEqual(user._id, same._id)
        self.assertEqual(user.email, same.email)

    def test_all(self):
        user = self.sisense.user.get(self.config['user_email'])
        users = self.sisense.user.all(email=self.config['user_email'])

        self.assertEqual(len(users), 1)
        self.assertEqual(users[0].email, user.email)
        self.assertEqual(users[0]._id, user._id)

        users = self.sisense.user.all()
        self.assertGreater(len(users), 1)

        for u in users:
            self.assertIsInstance(u, User)

    def test_update(self):
        user = self.sisense.user.get(self.config['user_email'])
        user.update(firstName='Dev', lastName='Team')

        user = self.sisense.user.get(self.config['user_email'])
        self.assertEqual(user.firstName, 'Dev')
        self.assertEqual(user.lastName, 'Team')

    def create(self):
        name = 'DevTeam'
        user = self.sisense.user.create(self.config['user_email'], userName=name, firstName=name + ' (1)')

        self.assertIsInstance(user, User)
        self.assertEqual(user.userName, name)
        self.assertEqual(user.firstName, name + ' (1)')

    def delete(self):
        user = self.sisense.user.get(self.config['user_email'])
        if user:
            user.delete()

            user = self.sisense.user.get(self.config['user_email'])
            self.assertEqual(user, None)



from .sisense_client import SisenseTestCase
from .permission import PermissionTestCase
from .hierarchy import HierarchyTestCase
from .permission import PermissionTestCase
from .datasecurity import DataSecurityTestCase
from .user import UserTestCase
from .group import GroupTestCase
from .dashboard import DashboardTestCase



from sisense.resource import Resource


class Connection(Resource):

    def all(self, **kwargs) -> list:
        """
        Get all connections.

        :param kwargs: Optional keyword arguments.
            - sort: (str) Field by which the result should be sorted.
            - skip: (int) Number the results to skip from the start. USe for paging.
            - limit: (int) Nuber of results to retrieve.
        :return: a list of Connection objects.
        """
        content = self._api.get('connection', query=kwargs)
        connections = [Connection(self._api, c) for c in content]

        return connections

    def get(self, oid: str) -> Resource:
        """
        Get the specified connection.

        :param oid: (str) Connections's ID.
        :return: (Connection)
        """
        content = self._api.get(f'connection/{oid}')
        return Connection(self._api, content)

    def create(self, **kwargs) -> Resource:
        """
        Create a new connection.

        :param kwargs: Keyword arguments.
            - owner: (str) User's ID.
            - provider: (str) Connection type: Excel, MySQL, MS SQL...
            - timeout: (int) Connection timeout.
            - refreshRate: (float) Optional. Just for live models.
            - timezone: (str) Optional. Timezone.
            - schema: (str) Database schema that will be used.
            - parameters: (dict) Other parameters such as UserName, Password, Database...
        :return: (Connection) The new connection.
        """
        content = self._api.post('connection', data=kwargs)
        return Connection(self._api, content)

    def update(self, **kwargs) -> Resource:
        """
        Update the current connection.

        :param kwargs: Keyword arguments.
            - owner: (str) User's ID.
            - provider: (str) Connection type: Excel, MySQL, MS SQL...
            - timeout: (int) Connection timeout.
            - refreshRate: (float) Optional. Just for live models.
            - timezone: (str) Optional. Timezone.
            - schema: (str) Database schema that will be used.
            - parameters: (dict) Other parameters such as UserName, Password, Database...
        :return: (Connection) The updated connection.
        """
        content = self._api.patch(f'connection/{self._id}', data=kwargs)
        return Connection(self._api, content)

    def delete(self):
        """Delete the current connection."""
        self._api.delete(f'connection/{self._id}')



from sisense.resource import Resource


class Build(Resource):

    PENDING = 'pending'
    BUILDING = 'building'
    DONE = 'done'
    CANCELLED = 'cancelled'
    FAILED = 'failed'

    def is_initialized(self) -> bool:
        """
        True, if build has a status.

        :return: (bool)
        """
        return self.status is not None

    def is_pending(self) -> bool:
        """
        True, if build has pending status.

        :return: (bool)
        """
        return self.status == self.PENDING

    def is_building(self) -> bool:
        """
        True, if build has building status.

        :return: (bool)
        """
        return self.status == self.BUILDING

    def is_done(self) -> bool:
        """
        True, if build is done.

        :return: (bool)
        """
        return self.status == self.DONE

    def was_cancelled(self) -> bool:
        """
        True, if build was cancelled.

        :return: (bool)
        """
        return self.status == self.CANCELLED

    def has_failed(self) -> bool:
        """
        True, if build has failed.

        :return: (bool)
        """
        return self.status == self.FAILED

    def is_finished(self) -> bool:
        """
        True, if build is not running nor pending.

        :return: (bool)
        """
        return self.is_done() or self.was_cancelled() or self.has_failed()

    def get(self, oid: str) -> Resource:
        """
        Get a build task by id.

        :param oid: (str) Build ID.

        :return: Build
        """
        content = self._api.get(f'builds/{oid}')
        return Build(self._api, content)

    def update(self):
        """Update this build information."""
        self.json = self.get(self.oid).json

    def all(self, datamodel_id: str = None, status: str = None) -> list:
        """
        Get a list of build tasks.

        :param datamodel_id: (str, default None) ID of Datamodel to filter results by.
        :param status: (str, default None) Build status to filter results by. Possible values: pending, building, done, failed, cancelled.

        :return: (list) List of Build objects.
        """
        query = {}
        if datamodel_id:
            query['datamodelId'] = datamodel_id

        if status:
            query['status'] = status

        content = self._api.get('builds', query=query)
        return [Build(self._api, task) for task in content]

    def start(self, datamodel_id: str, build_type: str = 'schema_changes', row_limit: int = None) -> Resource:
        """
        Start a new build.

        :param datamodel_id: (str) Datamodel ID.
        :param build_type: (str, default 'schema_changes') Type of build. Possible values: schema_changes, by_table, full
        :param row_limit: (int) Row limit.

        :return: Build
        """
        data = {'datamodelId': datamodel_id,
                'buildType': build_type,
                'rowLimit': row_limit}

        content = self._api.post('builds', data=data)
        return Build(self._api, content)

    def stop(self):
        """Cancel/stop this build."""
        self._api.delete(f'builds/{self.oid}')
        self.update()



from sisense.data import Datamodel, Build, Permission
from .test_case import TestCaseV2 as TestCase
from sisense.admin import User, Group
import unittest


class SisenseTestCase(TestCase):

    def test_datamodel(self):
        obj = self.sisense.datamodel
        self.assertEqual(type(obj), Datamodel)

    def test_build(self):
        obj = self.sisense.build
        self.assertEqual(type(obj), Build)

    def test_permission(self):
        obj = self.sisense.permission
        self.assertEqual(type(obj), Permission)

    def test_user(self):
        obj = self.sisense.user
        self.assertEqual(type(obj), User)

    def test_group(self):
        obj = self.sisense.group
        self.assertEqual(type(obj), Group)


if __name__ == '__main__':
    unittest.main()



from .api import API
from .analysis import *
from .admin import *
from .data import *


class Sisense:

    def __init__(self, host: str, token: str):
        """
        Manage API's entities.

        :param host: (str) Domain name server.
        :param token: (str) API's access token.
        """
        self._host = host
        self._token = token if token.startswith('Bearer ') else f'Bearer {token}'

    @property
    def datamodel(self) -> Datamodel:
        """Start point for datamodel objects."""
        api = self._api_v2()
        return Datamodel(api)

    @property
    def build(self) -> Build:
        """Start point for build objects."""
        api = self._api_v2()
        return Build(api)

    @property
    def elasticube(self) -> Elasticube:
        """Start point for elasticube objects."""
        api = self._api_v1()
        return Elasticube(api)

    @property
    def connection(self) -> Connection:
        """Start point for connection objects."""
        api = self._api_v1()
        return Connection(api)

    @property
    def permission(self) -> Permission:
        """Start point for permission objects."""
        api = self._api_v09()
        return Permission(api)

    @property
    def hierarchy(self) -> Hierarchy:
        """Start point for hierarchy objects."""
        api = self._api_v09()
        return Hierarchy(api)

    @property
    def datasecurity(self) -> DataSecurity:
        """Start point for datasecurity objects."""
        api = self._api_v09()
        return DataSecurity(api)

    @property
    def user(self) -> User:
        """Start point for user objects."""
        api = self._api_v1()
        return User(api)

    @property
    def group(self) -> Group:
        """Start point for group objects."""
        api = self._api_v1()
        return Group(api)

    @property
    def dashboard(self) -> Dashboard:
        """Start point for dashboard objects."""
        api = self._api_v1()
        return Dashboard(api)

    @property
    def folder(self) -> Folder:
        """Start point for folder objects."""
        api = self._api_v1()
        return Folder(api)

    @property
    def jaql(self) -> JAQL:
        """Start point for JAQL objects."""
        api = self._api_v09()
        return JAQL(api)

    def _api_v2(self):
        return API(self._host, 'v2', self._token)

    def _api_v1(self):
        return API(self._host, 'v1', self._token)

    def _api_v09(self):
        return API(self._host, '', self._token)



from .auth_model import AuthModel


class Auth(AuthModel):

    def login(self, username: str, password: str):
        self.logout()

        data = {'username': username, 'password': password}
        response = self._api.post('authentication/login', data)

        self._token = response['access_token']

    def logout(self):
        if self.is_logged():
            self._api.get('authentication/logout', headers={'authorization': self.token})
            self._reset_token()



from .resource import Resource
from .api import API


class CLI(Resource):

    def __init__(self, api: API, rjson: dict = None):
        """
        Super for any API's resource.

        :param api: (API) Used to make API's requests.
        :param rjson: (dict) Resource representation.
        """
        super(CLI, self).__init__(api, rjson)
        self._uri = 'cli/execute'

    @property
    def uri(self):
        return self._uri

    def execute(self, command: str):
        """
        Execute a command in the CLI.

        :param command: (str) Command without the "si" method. See https://documentation.sisense.com/docs/using-cli-commands.
        :return: (str) Content of the call
        """
        return self._api.post(self.uri, data=command, headers={'Content-type': 'text/plain', 'Accept': '*/*'})



from sisense.resource import Resource
from .user import User


class Group(Resource):

    def get(self, name: str = None, oid: str = None) -> Resource:
        """Get group by name. At least one parameter should be set.

        :param name: (str, default None) Group's name.
        :param oid: (str, default None) Group's ID.
        :return: Group, if found. None, otherwise.
        """
        if oid:
            content = self._api.get(f'groups/{oid}')
        elif name:
            content = self._api.get('groups', query={'name': name})
            content = content[0] if len(content) else None
        else:
            raise ValueError('At least one parameter should be set.')

        return Group(self._api, content) if content else None

    def all(self, **kwargs) -> list:
        """
        Get all groups.

        :param kwargs: Positional optional arguments used to filter users:
            - name: (str) Group's name.
            - mail: (str) Group's email.
            - roleId: (str) Role ID.
            - origin: ('ad', 'sisense') User's origin.
            - ids: (list) Groups' IDs.
            - fields: (list) Which fields to return.
            - sort: (str) Field in which the results should be sorted.
            - skip: (int) Number of results to skip from start.
            - limit: (int) Number of results returned.
            - expand: (list) Fields that should be expanded (substitutes their IDs with actual objects).
        :return: (list) of Group objects.
        """
        content = self._api.get('groups', query=kwargs)
        content = [Group(self._api, rjson) for rjson in content]

        return content

    def create(self, name: str) -> Resource:
        """
        Create a new group.

        :param name: (str) Group's name.
        :return: (Group) Newly created group.
        """
        data = {'name': name}
        content = self._api.post('groups', data=data)

        return Group(self._api, content)

    def delete(self):
        """Delete the current group."""
        self._api.delete(f'groups/{self._id}')

    def add_user(self, user: User):
        """
        Add a user to the current group.
        If user is already in the group, do nothing.

        :param user: (User) User to be added.
        """
        groups = user.groups if hasattr(user, 'groups') else []

        if self._id not in groups:
            groups.append(self._id)
            user.update(groups=groups)

    def remove_user(self, user: User):
        """
        Remove a user from the current group.
        If user isn't in the group, do nothing.

        :param user: (User) User to be removed.
        """
        groups = user.groups if hasattr(user, 'groups') else []

        if self._id in groups:
            groups = list(set(groups) - {self._id})
            user.update(groups=groups)



from .test_case import TestCaseV1 as TestCase
from sisense.analysis import Folder
import unittest


class FolderTestCase(TestCase):

    def test_get(self):
        folder = self.sisense.folder
        results = folder.all()

        oid = results[0].oid

        one_result = folder.get(oid)
        self.assertIsInstance(one_result, Folder)
        self.assertEqual(one_result.oid, oid)
        self.assertEqual(one_result.name, results[0].name)

    def test_all(self):
        folder = self.sisense.folder
        results = folder.all()

        self.assertGreater(len(results), 1)

        for f in results:
            self.assertIsInstance(f, Folder)

    def create(self):
        folder = self.sisense.folder
        n_folders = len(folder.all())

        parent = folder.create(self.config['folder'])
        child = folder.create(self.config['folder'] + ' Child', parent.oid)

        folders = folder.all()

        self.assertEqual(n_folders + 2, len(folders))
        self.assertEqual(parent.name, self.config['folder'])
        self.assertEqual(child.parentId, parent.oid)

    def delete(self):
        folders = self.sisense.folder.all()

        i = 0
        for folder in folders:
            if folder.name == self.config['folder']:
                folder.delete()
                i = i + 2

        new_folders = self.sisense.folder.all()
        self.assertEqual(len(folders) - i, len(new_folders))


if __name__ == '__main__':
    unittest.main()



from urllib.parse import quote, urljoin
from .utils import is_json
import requests
import os


class API:

    def __init__(self, host: str, version: str, token: str = None):
        """
        Manage server's requests and responses.

        :param host: (str) Domain name server.
        :param version: (str) API's version. Ex.: 'v1'.
        :param token: (str, default None) API's access token.
        """
        self._host = host
        self._version = version
        self._token = token

        self._url = os.path.join(host, 'api', version) + '/'

    @property
    def host(self) -> str:
        """API's domain name server."""
        return self._host

    @property
    def version(self) -> str:
        """API's version."""
        return self._version

    def url(self, uri: str = '') -> str:
        """
        Get API's full URL.

        :param uri: (str, default '') Resource identifier.
        :return: (str) Full url.
        """
        uri = quote(uri)
        return urljoin(self._url, uri)

    def get(self, uri: str, query: dict = None, headers: dict = None) -> dict:
        """
        Retrieve the specified resource.

        :param uri: (str) Resource identifier.
        :param query: (dict, default None) GET query parameters.
        :param headers: (dict, default None) Request headers.

        :return: (dict) Response
        """
        return self._request('GET', uri, headers=headers, params=query)

    def post(self, uri: str, data = None, query: dict = None, headers: dict = None) -> dict:
        """
        Create a resource in the specified collection.

        :param uri: (str) Resource identifier.
        :param data: (dict or str, default None) Resource representation as json or a command as string.
        :param query: (dict, default None) GET query parameters.
        :param headers: (dict, default None) Request headers.

        :return: (dict) Response
        """
        if type(data) is dict:
            return self._request('POST', uri, headers=headers, json=data, params=query)

        return self._request('POST', uri, headers=headers, data=data, params=query)

    def put(self, uri: str, data: dict = None, headers: dict = None) -> dict:
        """
        Replace the specified resource.

        :param uri: (str) Resource identifier.
        :param data: (dict, default None) Resource representation as json.
        :param headers: (dict, default None) Request headers.

        :return: (dict) Response
        """
        return self._request('PUT', uri, headers=headers, json=data)

    def patch(self, uri: str, data: dict = None, headers: dict = None) -> dict:
        """
        Update the specified resource without replacing it.

        :param uri: (str) Resource identifier.
        :param data: (dict, default None) Resource representation as json.
        :param headers: (dict, default None) Request headers.

        :return: (dict) Response
        """
        return self._request('PATCH', uri, headers=headers, json=data)

    def delete(self, uri: str, query: dict = None, headers: dict = None) -> dict:
        """
        Delete the specified resource.

        :param uri: (str) Resource identifier.
        :param query: (dict, default None) GET query parameters.
        :param headers: (dict, default None) Request headers.

        :return: (dict) Response
        """
        return self._request('DELETE', uri, headers=headers, params=query)

    def download(self, uri: str, filepath: str, query: dict = None, headers: dict = None):
        """
        Download file from stream.

        :param uri: (str) Resource identifier.
        :param filepath: (str) Where to save the downloaded file, including file's name.
        :param query: (dict, default None) GET query parameters.
        :param headers: (dict, default None) Request headers.
        """
        path = self.url(uri)
        headers = self._headers(headers)
        headers.update({'Accept': '*/*'})

        with requests.get(path, params=query, headers=headers, stream=True, verify=False) as response:
            self._handle_request_error(response)

            with open(filepath, 'wb') as file:
                [file.write(chunk) for chunk in response.iter_content(chunk_size=None, decode_unicode=True) if chunk]

    def upload(self, uri: str, file: dict, query: dict = None, headers: dict = None):
        """
        Upload a file.

        :param uri: (str) Resource identifier.
        :param file: (dict) {key: file-like-object File to be uploaded}.
        :param query: (dict, default None) GET query parameters.
        :param headers: (dict, default None) Request headers.
        """
        path = self.url(uri)
        headers = self._headers(headers)
        headers.update({'Accept': '*/*'})
        headers.update({'Content-type': 'multipart/form-data'})

        response = requests.post(path, files=file, params=query, headers=headers, verify=False)
        self._handle_request_error(response)

    def _headers(self, other: dict) -> dict:
        headers = {'authorization': self._token} if self._token else {}

        headers.update({'Accept': 'application/json'})
        # headers.update({'Content-type': 'application/json'})

        headers.update(other if other else {})

        return headers

    def _request(self, method: str, uri: str, **kwargs) -> dict:
        requests.packages.urllib3.disable_warnings()

        path = self.url(uri)
        kwargs['headers'] = self._headers(kwargs['headers'])
        kwargs['verify'] = False

        response = requests.request(method, path, **kwargs)
        self._handle_request_error(response)

        if len(response.text):
            content = response.json() if is_json(response.text) else {'message': response.text}
        else:
            content = {}

        return content

    def _handle_request_error(self, response):
        response.raise_for_status()

    # TODO: keep requests log



import json
import os


def is_windows() -> bool:
    """Check if OS is Windows. Otherwise, assume it is linux.

    :return: (bool) True, if OS is Windows. False, otherwise.
    """
    return os.name == 'nt'


def is_json(value: str) -> bool:
    """Check if string is a JSON parsable.

    :param value: (str) Value to check.
    :return: (bool) True, if value is a valid JSON string. False, otherwise.
    """
    try:
        _ = json.loads(value)
    except ValueError:
        return False

    return True



from .test_case import TestCaseV1 as TestCase
from sisense.data import Elasticube
import unittest


class ElasticubeTestCase(TestCase):

    def test_all(self):
        results = self.sisense.elasticube.all()

        self.assertIsInstance(results, list)
        self.assertGreater(len(results), 0)

        for cube in results:
            self.assertIsInstance(cube, Elasticube)

    def test_get(self):
        cube = self.sisense.elasticube.get(self.config['elasticube'])

        self.assertIsInstance(cube, Elasticube)
        self.assertEqual(cube.title, self.config['elasticube'])


if __name__ == '__main__':
    unittest.main()



from sisense.api import API


class AuthModel:

    def __init__(self, host: str, username: str, password: str):
        """
        Authentication model for Sisense API.

        :param host: (str) Domain name server.
        :param username: (str) User's login name.
        :param password: (str) User's password.
        """
        self._token = ''
        self._api = API(host, 'v1')

        self.login(username, password)

    @property
    def token(self) -> str:
        """
        Get access token.

        :return: (str)
        """
        return f"Bearer {self._token}"

    def login(self, username: str, password: str):
        """
        Get an access token using the specified credentials.

        :param username: (str) User's login name.
        :param password: (str) User's password.
        """
        raise NotImplementedError

    def logout(self):
        """Revoke access token."""
        raise NotImplementedError

    def is_logged(self) -> bool:
        """True, if user is logged. False, otherwise."""
        return bool(len(self._token))

    def _reset_token(self):
        self._token = ''

    def __del__(self):
        self.logout()



from sisense.resource import Resource
import pandas as pd
import json
import io


class JAQL(Resource):

    def to_csv(self, datasource, metadata: list) -> pd.DataFrame:
        """
        Execute JAQL and return the result as CSV.

        :param datasource: (str or dict) Elasticube name or datasource representation as JSON.
        :param metadata: (list) A list of JAQL representations as dict. See https://sisense.dev/reference/jaql/ for details.
        :return: (pandas.DataFrame) Data as CSV represent by a pandas.DataFrame.
        """
        data = [('data', json.dumps({'datasource': datasource, 'metadata': metadata}))]
        headers = {'Accept': '*/*'}

        response = self._api.post('datasources/x/jaql/csv', data=data, headers=headers)

        data = io.StringIO(response['message'])
        dataframe = pd.read_csv(data, header=0, index_col=None)

        return dataframe



from sisense.resource import Resource
from sisense.api import API


class Hierarchy(Resource):

    def __init__(self, api: API, rjson: dict = None, elasticube_name: str = None):
        super().__init__(api, rjson)
        self._elasticube = elasticube_name

    def all(self, elasticube: str = None) -> list:
        """
        Get elasticube's hierarchies.

        :param elasticube: (str, default None) Elasticube's name. If None, use self.elasticube.
        :return: a list of Hierarchy objects
        """
        elasticube = elasticube if elasticube else self._elasticube
        query = {'elasticube': elasticube, 'server': 'localhost'}

        content = self._api.get(f'elasticubes/hierarchies', query=query)
        hierarchies = [Hierarchy(self._api, h, elasticube) for h in content]

        return hierarchies

    def get(self, oid: str, elasticube: str = None) -> Resource:
        """
        Get the specified hierarchy.

        :param oid: (str) Hierarchy's ID.
        :param elasticube: (str, default None) Elasticube's name. If None, use self.elasticube.
        :return: (Hierarchy) if found. Otherwise, None.
        """
        for hierarchy in self.all(elasticube):
            if hierarchy._id == oid:
                return hierarchy

        return None

    def create(self, title: str, levels: list, always_included: bool, elasticube: str = None) -> Resource:
        """
        Create a new hierarchy.

        :param title: (str) Hierarchy's title.
        :param levels: (list) List of dict {'title': str, 'table': str, 'column': str, 'datatype': str, 'dim': str, index: int}.
        :param always_included: (bool) Whether to always include the hierarchy on widget.
        :param elasticube: (str, default None) Elasticube's name. If None, use self.elasticube.
        :return: (Hierarchy) The new hierarchy.
        """
        elasticube = elasticube if elasticube else self._elasticube
        data = {'title': title, 'levels': levels, 'alwaysIncluded': always_included}

        content = self._api.post(f'elasticubes/localhost/{elasticube}/hierarchies', data=data)
        return Hierarchy(self._api, content, elasticube)

    def delete(self):
        """Delete the current hierarchy."""
        self._api.delete(f'elasticubes/localhost/{self._elasticube}/hierarchies/{self._id}')



from .auth_model import AuthModel


class OAuth(AuthModel):

    def login(self, username: str, password: str):
        raise NotImplementedError  # TODO: implement login

    def logout(self):
        raise NotImplementedError  # TODO: implement logout



from .test_case import TestCaseV1 as TestCase
from sisense.admin import Group


class GroupTestCase(TestCase):

    def test_get(self):
        group = self.sisense.group.get(self.config['group_name'])
        self.assertEqual(group.name, self.config['group_name'])

        same = self.sisense.group.get(oid=group._id)
        self.assertEqual(group._id, same._id)
        self.assertEqual(group.name, same.name)

    def test_all(self):
        group = self.sisense.group.get(self.config['group_name'])
        groups = self.sisense.group.all(name=self.config['group_name'])

        self.assertEqual(len(groups), 1)
        self.assertEqual(groups[0].name, group.name)
        self.assertEqual(groups[0]._id, group._id)

        groups = self.sisense.group.all()
        self.assertGreater(len(groups), 1)

        for g in groups:
            self.assertIsInstance(g, Group)

    def test_add_user(self):
        group = self.sisense.group.get(self.config['group_name'])
        user = self.sisense.user.create(self.config['user_email'])

        group.add_user(user)
        user = self.sisense.user.get(self.config['user_email'])

        user.delete()
        self.assertEqual(user.groups[0], group._id)

    def test_remove_user(self):
        group = self.sisense.group.get(self.config['group_name'])
        user = self.sisense.user.create(self.config['user_email'])

        group.add_user(user)
        user = self.sisense.user.get(self.config['user_email'])
        self.assertEqual(user.groups[0], group._id)

        group.remove_user(user)

        user = self.sisense.user.get(self.config['user_email'])
        user.delete()

        self.assertEqual(len(user.groups), 0)

    def create(self):
        group = self.sisense.group.create(self.config['group_name'])

        self.assertIsInstance(group, Group)
        self.assertEqual(group.name, self.config['group_name'])

    def delete(self):
        group = self.sisense.group.get(self.config['group_name'])
        if group:
            group.delete()

            group = self.sisense.group.get(self.config['group_name'])
            self.assertEqual(group, None)



from .api import API
import json


class Resource:

    def __init__(self, api: API, rjson: dict = None):
        """
        Super for any API's resource.

        :param api: (API) Used to make API's requests.
        :param rjson: (dict) Resource representation.
        """
        # If you add a new attribute, always remember to include it on __setattr__
        self._api = api
        self.json = rjson if rjson else {}
        self.fix_shares()

    def new(self, rjson: dict) -> object:
        """
        Create a new resource with different properties, but same API.

        :param rjson: (dict) Resource representation.
        :return: (Resource) The new resource
        """
        return self.__class__(self._api, rjson)

    def save(self, filepath: str):
        """
        Save the object representation as a .json file.

        :param filepath: (str) Relative/absolute path, including filename.
        """
        with open(filepath, 'w') as file:
            json.dump(self.json, file)

    def load(self, filepath: str) -> object:
        """
        Load the object representation.

        :param filepath: (str) Relative/absolute path to a .json file.
        """
        with open(filepath, 'r') as file:
            rjson = json.load(file)

        return self.new(rjson)

    def fix_shares(self) -> object:
        """
        When API returns some resource with shares, each share has a field called 'partyId'.
        However, when adding or updating this resource, the API only accepts the field 'party'.
        So, this method replaces 'partyId' with 'party'.

        :return: (Resource) The updated resource.
        """
        if hasattr(self, 'shares'):
            for i, share in enumerate(self.shares):
                if 'partyId' in share:
                    share['party'] = share['partyId']

                share.pop('partyId', None)
                share.pop('partyName', None)

                self.shares[i] = share

        elif hasattr(self, 'partyId'):
            self.json['party'] = self.json['partyId']
            self.json.pop('partyId')

        return self

    def __setattr__(self, key, value):
        if key in self.__dict__ or key in ['json', '_api', '_elasticube']:
            self.__dict__[key] = value
        else:
            self.__dict__['json'][key] = value

    def __getattr__(self, item):
        if item in self.__dict__:
            return self.__dict__[item]

        if item in self.__dict__['json']:
            return self.__dict__['json'][item]

        raise AttributeError(f'"{item}" does not exist in {type(self)}.')

    def __repr__(self):
        return str(self.json)



from .test_case import TestCaseV09 as TestCase
from sisense.data import DataSecurity
import unittest
import json


class DataSecurityTestCase(TestCase):

    def test_all(self):
        datasecurity = self.sisense.datasecurity

        results = datasecurity.all(elasticube=self.config['elasticube'])
        ids = [r._id for r in self.rules]

        self.assertIsInstance(results, list)
        self.assertEqual(len(results), len(self.rules))

        for rule in results:
            self.assertIsInstance(rule, DataSecurity)
            self.assertIn(rule._id, ids)

        results = datasecurity.all(self.rules[-1].table, self.rules[-1].column, self.config['elasticube'])
        for rule in results:
            self.assertEqual(self.rules[-1].table, rule.table)
            self.assertEqual(self.rules[-1].column, rule.column)

    def test_get(self):
        rule = self.sisense.datasecurity.get(self.rules[0]._id, self.config['elasticube'])

        self.assertIsInstance(rule, DataSecurity)
        self.assertEqual(rule._id, self.rules[0]._id)

    def test_update(self):
        for r in self.rules:
            if hasattr(r, 'members') and len(r.members):
                rule = r
                break

        rule.exclusionary = True
        rule.members.append('California')

        new_rule = rule.update()

        self.assertTrue(new_rule.exclusionary)
        self.assertEqual(len(new_rule.members), len(rule.members))
        self.assertIn('California', new_rule.members)

    def test_delete_all(self):
        datasecurity = self.sisense.datasecurity

        datasecurity.delete_all('DimProducts', 'ProductName', self.config['elasticube'])
        this_results = datasecurity.all('DimProducts', 'ProductName', elasticube=self.config['elasticube'])
        other_results = datasecurity.all(elasticube=self.config['elasticube'])

        self.assertEqual(len(this_results), 0)
        self.assertNotEqual(len(other_results), len(self.rules))

    def create(self):
        datasecurity = self.sisense.datasecurity

        with open('tests/support_files/datasecurity.json', 'r') as file:
            rules = json.load(file)

        for rule in rules:
            rule.pop('elasticube', None)
            datasecurity.create(**rule, elasticube=self.config['elasticube'])

        self.rules = datasecurity.all(elasticube=self.config['elasticube'])
        self.assertEqual(len(self.rules), len(rules))

    def delete(self):
        datasecurity = self.sisense.datasecurity
        results = datasecurity.all(elasticube=self.config['elasticube'])
        [rule.delete() for rule in results]

        new_results = datasecurity.all(elasticube=self.config['elasticube'])
        self.assertEqual(len(new_results), 0)


if __name__ == '__main__':
    unittest.main()



from .test_case import TestCaseV09 as TestCase
from sisense.data import Permission
import unittest
import json


class PermissionTestCase(TestCase):

    def test_all(self):
        results = self.sisense.permission.all(elasticube=self.config['elasticube'])
        ids = [p.party for p in self.permissions]

        self.assertIsInstance(results, list)
        self.assertEqual(len(results), len(self.permissions))

        for p in results:
            self.assertIsInstance(p, Permission)
            self.assertIn(p.party, ids)

    def test_get(self):
        p = self.sisense.permission.get(self.permissions[-1].party, self.config['elasticube'])

        self.assertIsInstance(p, Permission)
        self.assertEqual(p.party, self.permissions[-1].party)

    def test_delete(self):
        permission = self.sisense.permission

        p = permission.get(self.permissions[-1].party, self.config['elasticube'])
        p.delete()

        results = permission.all(self.config['elasticube'])
        ids = [p.party for p in results]

        self.assertEqual(len(results), len(self.permissions) - 1)
        self.assertNotIn(p.party, ids)

    def create(self):
        permission = self.sisense.permission

        with open('tests/support_files/permission.json', 'r') as file:
            permissions = json.load(file)

        for p in permissions:
            permission.create(p['party'], p['type'], p['permission'], self.config['elasticube'])

        self.permissions = permission.all(elasticube=self.config['elasticube'])
        self.assertEqual(len(self.permissions), len(permissions))

    def delete(self):
        permission = self.sisense.permission
        permission.delete_all(self.config['elasticube'])

        results = permission.all(self.config['elasticube'])
        self.assertEqual(len(results), 1)


if __name__ == '__main__':
    unittest.main()



from sisense.resource import Resource


class Elasticube(Resource):

    def all(self) -> list:
        """
        Get all elasticubes.

        :return: (list) of Elasticube objects.
        """
        content = self._api.get('elasticubes/getElasticubes')
        return [Elasticube(self._api, rjson) for rjson in content]

    def get(self, name: str) -> Resource:
        """
        Get elasticube.

        :param name: (str) Elasticube's name.
        :return: (Elasticube)
        """
        content = self._api.get(f'elasticubes/servers/next/localhost/{name}')
        return Elasticube(self._api, content)



from sisense.resource import Resource
from sisense.cli import CLI
from sisense.api import API
import json


class Datamodel(Resource):

    def __init__(self, api: API, rjson: dict = None):
        """
        Super for any API's resource.

        :param api: (API) Used to make API's requests.
        :param rjson: (dict) Resource representation.
        """
        super(Datamodel, self).__init__(api, rjson)
        self._cli = CLI(api)

    def list(self):
        """
        Get the following information for each elasticube:
            - instance: Query instance
            - id: elasticube's ID
            - name: elasticube's name
            - runtime_status: RUNNING or STOPPED
            - index_size: LONG or SHORT
            - path: path to the elasticube data on Linux
            - shadow_path: (?)
            - next_path: in case the elasticube is divided into two paths
            - last_failure_message: if build failed, shows the last message

        :return: (list) a list of Datamodel objects
        """
        response = self._cli.execute('elasticubes list')

        lines = response['message'].split('\n')
        keys = [key.strip().lower().replace(' ', '_') for key in lines[1].split('|')][1:-1]

        result = []
        for line in lines[3:-1]:
            values = [value.strip() if len(value.strip()) else None for value in line.split('|')][1:-1]
            rjson = dict(zip(keys, values))
            datamodel = Datamodel(self._api, rjson)
            result.append(datamodel)

        return result

    def get(self, oid: str = None, title: str = None) -> Resource:
        """
        Get datamodel by ID. At least one of the parameters should be set.

        :param oid: (str, default None) Datamodel's ID to search for.
        :param title: (str, default None) Datamodel's title to search for. If oid is set, title is ignored.

        :return: Datamodel
        """
        if not oid and not title:
            raise ValueError('At least one parameter should be set.')

        if oid:
            content = self._api.get(f'datamodels/{oid}/schema')
        else:
            query = {'title': title, 'limit': 1}
            content = self._api.get('datamodels/schema', query=query)

        return Datamodel(self._api, content)

    def create(self, title: str, server: str = 'LocalHost', ctype: str = 'extract') -> Resource:
        """
        Create a new datamodel.

        :param title: (str) Datamodel's title.
        :param server: (str, default 'LocalHost') Server in which the datamodel should be created.
        :param ctype: (str, default 'extract') Creation type.

        :return: (Datamodel) The new datamodel.
        """
        data = {'title': title, 'server': server, 'type': ctype}
        content = self._api.post('datamodels', data=data)
        return Datamodel(self._api, content)

    def clone(self, title: str) -> Resource:
        """
        Clone this datamodel.

        :param title: (str) Datamodel's title.

        :return: (Datamodel) The new datamodel.
        """
        data = {'title': title}
        content = self._api.post(f'datamodels/{self.oid}/clones', data=data)
        return Datamodel(self._api, content)

    def delete(self):
        """Delete this data model."""
        self._api.delete(f'datamodels/{self.oid}')

    def start(self):
        """Start this data model."""
        name = self.name if 'name' in self.json else self.title
        self._cli.execute(f'elasticubes start -name "{name}"')

    def stop(self):
        """Stop this data model."""
        name = self.name if 'name' in self.json else self.title
        self._cli.execute(f'elasticubes stop -name "{name}"')

    def do_export(self, filepath: str, full: bool = False):
        """
        Export the datamodel.

        :param filepath: (str) Where to save the downloaded file, including file's name.
        :param full: (bool, default False) If true, export datamodel with schema and data. Otherwise, only export datamodel's schema.
        """
        self._download_full(filepath) if full else self._download_schema(filepath)

    def do_import(self, title: str, filepath: str, full: bool = False) -> Resource:
        """
        Import a datamodel.

        :param title: (str) New datamodel's title.
        :param filepath: (str) Where to get data for import, including file's name.
        :param full: (bool, default False) If true, import datamodel with schema and data. Otherwise, only import datamodel's schema.

        :return: (Datamodel) The new datamodel.
        """
        self._upload_full(title, filepath) if full else self._upload_schema(title, filepath)
        return self.get(title=title)

    def _download_schema(self, filepath: str):
        query = {'datamodelId': self.oid, 'type': 'schema-latest'}
        content = self._api.get('datamodel-exports/schema', query=query)

        with open(filepath, 'w') as file:
            json.dump(content, file)

    def _download_full(self, filepath):
        query = {'datamodelId': self.oid}
        self._api.download('datamodel-exports/stream/full', filepath, query=query)

    def _upload_schema(self, title: str, filepath: str):
        with open(filepath, 'r') as file:
            schema = json.load(file)

        query = {'newTitle': title}
        self._api.post('datamodel-imports/schema', data=schema, query=query)

    def _upload_full(self, title: str, filepath: str):
        query = {'newTitle': title}
        data = {'fileToUpload': open(filepath, 'rb')}
        self._api.upload('datamodel-imports/stream/full', file=data, query=query)





from sisense.resource import Resource
from .widget import Widget
import json


class Dashboard(Resource):

    @property
    def widgets(self) -> list:
        """
        Get dashboard's widgets.

        :return: (list) A list of widgets.
        """
        dashboard = self._expanded()
        return [Widget(self._api, content) for content in dashboard.json['widgets']]

    def all(self, **kwargs) -> list:
        """
        Get all dashboards.

        :param kwargs: Keywords optional arguments.
            - parentFolder: (str) Folder ID.
            - name: (str) Dashboard's name.
            - datasourceTitle: (str) Elasticube title used as datasource.
            - datasourceAddress: (str) Elasticube address.
            - fields: (list) List of fields to retrieve for each dashboard.
            - expand: (list) List of fields to be expanded (convert IDs to the actual object).
        :return: (list) List of Dashboard objects.
        """
        content = self._api.get('dashboards', query=kwargs)
        dashboards = [Dashboard(self._api, rjson) for rjson in content]
        return dashboards

    def get(self, oid: str = None, name: str = None, folder: str = None, expand: list = None) -> Resource:
        """
        Get the specified dashboard. At least, one of the parameters must be set.
        If found more than one dashboard with the same name, the first one is returned.

        :param oid: (str, default None) Dashboard's ID.
        :param name: (str, default None) Dashboard's name.
        :param folder: (str, default None) Parent folder's ID. Used when 'name' is set.
        :param expand: (list, default None) List of fields that should be expanded (replace their IDs with actual objects). May be nested using the resource.subResource format.
        :return: (Dashboard) Dashboard, if found. None, otherwise.
        """
        if not oid and not name:
            raise ValueError('At least, one of the parameters must be set. Both oid and name are None.')

        expand = ','.join(expand) if type(expand) is list else []
        query = {'expand': expand}

        if oid:
            content = self._api.get(f'dashboards/{oid}', query=query)
        else:
            query.update({'name': name, 'parentFolder': folder})
            content = self._api.get('dashboards', query=query)
            content = content[0] if len(content) else None

        if not content:
            return None

        result = Dashboard(self._api, content)
        return result

    def get_shares(self) -> list:
        """Get shares for the current dashboard."""
        content = self._api.get(f'dashboards/{self.oid}/shares')
        return content

    def update(self, **kwargs):
        """
        Update the current dashboard information according to the specified parameters.
        For more information, check "PATCH /dashboards/{id}" on https://sisense.dev/reference/rest/v1.html.
        """
        data = kwargs
        data['oid'] = self.oid

        content = self._api.patch(f'dashboards/{self.oid}', data=data)
        self.json = content

    def exists(self, oid: str = None) -> bool:
        """
        Check whether a specific dashboard exists.

        :param oid: (str, default self.oid) Dashboard's ID.
        :return: (bool) True, if dashboard exists.
        """
        oid = oid if oid else self.oid
        content = self._api.get(f'dashboards/{oid}/exists')
        return content['exists']

    def do_import(self, filepath: str, action: str = 'duplicate', folder: str = None) -> Resource:
        """
        Import dashboard from file in filepath.

        :param filepath: (str) Relative/absolute path to a .dash file.
        :param action: (str, default 'duplicate') Determines if the existing dashboard should be overwritten. Possible values : skip, overwrite, duplicate.
        :param folder: (str, default None) Folder's ID where the dashboard should be imported.
        :return: (Dashboard) The new dashboard.
        """
        with open(filepath, 'r') as file:
            rjson = json.load(file)

        query = {'action': action, 'importFolder': folder}
        content = self._api.post('dashboards/import/bulk', query=query, data=[rjson])

        return Dashboard(self._api, content['succeded'][0])

    def do_export(self, filepath: str, filetype: str = 'dash', **kwargs):
        """
        Export the current dashboard.

        :param filepath: (str) Where to save the file including file's name and extension.
        :param filetype: (str, default 'dash') Type of export. Possible values: dash.

        For more details on other parameters, check:
        <GET /dashboards/{id}/export/*> on https://sisense.dev/reference/rest/v1.html.

        PNG and PDF export is currently not supported.
        """
        content = self._api.get(f'dashboards/{self.oid}/export/{filetype}', query=kwargs)
        dashboard = Dashboard(self._api, content)
        dashboard.save(filepath)

    def publish(self):
        """Publish the current dashboard."""
        self._api.post(f'dashboards/{self.oid}/publish')

    def delete(self):
        """Delete the current dashboard."""
        self._api.delete(f'dashboards/{self.oid}')

    def _expanded(self) -> Resource:
        """
        Get dashboard's expanded information, including widgets.

        :return: (Dashboard) The expanded dashboard.
        """
        content = self._api.get(f'dashboards/{self.oid}/export/dash')
        return Dashboard(self._api, content)



from sisense.resource import Resource


class Folder(Resource):

    def get(self, oid: str) -> Resource:
        """
        Get a specific folder.

        :param oid: (str) Folder's ID.
        :return: (Folder)
        """
        content = self._api.get(f'folders/{oid}')
        return Folder(self._api, content)

    def all(self) -> list:
        """
        Get all folders.

        :return: (list) List of folder objects.
        """
        content = self._api.get('folders')
        results = [Folder(self._api, rjson) for rjson in content]
        return results

    def create(self, name: str, parent: str = None) -> Resource:
        """
        Create a new folder.

        :param name: (str) Folder's name.
        :param parent: (str, default None) Parent folder's ID.
        :return: (Folder) The new folder.
        """
        data = {'name': name}
        if parent:
            data['parentId'] = parent

        content = self._api.post('folders', data=data)
        return Folder(self._api, content)

    def delete(self):
        """Delete the current folder."""
        self._api.delete(f'folders/{self.oid}')



from .test_case import TestCaseV1 as TestCase
from sisense.analysis import Dashboard
import unittest
import json
import os


class DashboardTestCase(TestCase):

    def test_all(self):
        samples = self.sisense.dashboard.all(datasourceTitle=self.config['elasticube'])

        self.assertIsInstance(samples, list)

        for sample in samples:
            self.assertIsInstance(sample, Dashboard)
            self.assertEqual(sample.datasource['title'], self.config['elasticube'])

    def test_get(self):
        dashboard = self.sisense.dashboard
        sample = dashboard.get(name=self.config['dashboard'])

        self.assertIsInstance(sample, Dashboard)
        self.assertEqual(sample.title, self.config['dashboard'])

        same_sample = dashboard.get(oid=sample.oid)

        self.assertEqual(same_sample.title, sample.title)
        self.assertEqual(same_sample.oid, sample.oid)

        sample = dashboard.get(name='Non existing dashboard')
        self.assertEqual(sample, None)

    def test_exists(self):
        dashboard = self.sisense.dashboard
        sample = dashboard.get(name=self.config['dashboard'])

        self.assertTrue(sample.exists())
        self.assertTrue(dashboard.exists(sample.oid))
        self.assertFalse(dashboard.exists('5f624193993ff1002d46e06d'))

    def test_get_shares(self):
        dashboard = self.sisense.dashboard
        sample = dashboard.get(name=self.config['dashboard'])

        shares = sample.get_shares()
        self.assertEqual(len(shares), 1)

    def test_update(self):
        dashboard = self.sisense.dashboard
        sample = dashboard.get(name=self.config['dashboard'])

        new_title = 'New title'
        sample.update(title=new_title)

        sample = dashboard.get(oid=sample.oid)
        self.assertEqual(sample.title, new_title)

        new_title = self.config['dashboard']
        sample.update(title=new_title)

    def test_do_export(self):
        filepath = 'test_export.dash'

        dashboard = self.sisense.dashboard
        sample = dashboard.get(name=self.config['dashboard'])
        sample.do_export(filepath)

        same_sample = sample.load(filepath)

        self.assertEqual(sample.oid, same_sample.oid)
        self.assertEqual(sample.title, same_sample.title)
        self.assertEqual(sample.parentFolder, same_sample.parentFolder)

        os.system(f'rm {filepath}')

    def create(self):
        dashboard = self.sisense.dashboard
        new_dash = dashboard.do_import('tests/support_files/dashboard.dash', 'overwrite')

        with open('tests/support_files/dashboard.dash', 'r') as file:
            rjson = json.load(file)

        same_dash = dashboard.get(oid=new_dash.oid)
        self.assertEqual(same_dash.oid, rjson['oid'])
        self.assertEqual(same_dash.title, rjson['title'])

    def delete(self):
        dashboard = self.sisense.dashboard
        sample = dashboard.get(name=self.config['dashboard'])

        if sample:
            sample.delete()

        sample = dashboard.get(name=self.config['dashboard'])
        self.assertEqual(sample, None)


if __name__ == '__main__':
    unittest.main()



from sisense import Sisense
import unittest
import json


def _load_config(version: str) -> dict:
    with open(f'tests/support_files/config/api_{version}.json', 'r') as file:
        return json.load(file)


class TestCase(unittest.TestCase):

    def create(self):
        pass

    def delete(self):
        pass

    def setUp(self) -> None:
        super().setUp()
        self.delete()
        self.create()

    def tearDown(self) -> None:
        super().tearDown()
        self.delete()


class TestCaseV09(TestCase):

    def setUp(self):
        self.config = _load_config('v09')
        self.sisense = Sisense(self.config['host'], self.config['token'])

        super().setUp()


class TestCaseV1(TestCase):

    def setUp(self):
        self.config = _load_config('v1')
        self.sisense = Sisense(self.config['host'], self.config['token'])

        super().setUp()


class TestCaseV2(TestCase):

    def setUp(self):
        self.config = _load_config('v2')
        self.sisense = Sisense(self.config['host'], self.config['token'])

        super().setUp()



from sisense.resource import Resource
from sisense.api import API


class DataSecurity(Resource):

    def __init__(self, api: API, rjson: dict = None, elasticube_name: str = None):
        super().__init__(api, rjson)
        self._elasticube = elasticube_name

    def all(self, table: str = None, column: str = None, elasticube: str = None) -> list:
        """
        Get elasticube's data security rules. If 'table' and 'column' are specified, get rules for that specific column.

        :param table: (str, default None) Datatable's name.
        :param column: (str, default None) Column's name.
        :param elasticube: (str, default None) Elasticube's name. If None, use self.elasticube.
        :return: (list) of DataSecurity objects
        """
        elasticube = elasticube if elasticube else self._elasticube
        tbl_col = f'/{table}/{column}' if table and column else ''

        content = self._api.get(f'elasticubes/localhost/{elasticube}/datasecurity{tbl_col}')

        results = [DataSecurity(self._api, rjson, elasticube) for rjson in content]
        return results

    def get(self, oid: str, elasticube: str = None) -> Resource:
        """
        Get the specified data security rule.

        :param oid: (str) Rule's ID.
        :param elasticube: (str, default None) Elasticube's name. If None, use self.elasticube.
        :return: (DataSecurity) if found. Otherwise, None.
        """
        for rule in self.all(elasticube=elasticube):
            if rule._id == oid:
                return rule

        return None

    def create(self, table: str, column: str, datatype: str, shares: list, elasticube: str = None, **kwargs) -> Resource:
        """
        Create a new data security rule.

        :param table: (str) Datatable's name.
        :param column: (str) Column's name.
        :param datatype: (str) Type of data. For example: 'text'.
        :param shares: (list) List of shares dict {'party': <user/group id>, 'type': <'user' or 'group'>}
        :param members: (list, default None) Values considered. For example: if column is CountryName, then members can be a list of country names to allow or deny.
        :param all_members: (bool, default True) Whether to apply the rules for all members. If members is set, this parameter is ignored.
        :param exclusionary: (bool, default True) Allow (False) or deny (True) members.
        :param elasticube: (str, default None) Elasticube's name. If None, use self.elasticube.
        :return: (DataSecurity) The new data security rule.
        """
        elasticube = elasticube if elasticube else self._elasticube
        data = self._payload(table, column, datatype, shares, **kwargs)

        content = self._api.post(f'elasticubes/localhost/{elasticube}/datasecurity', data=[data])
        result = DataSecurity(self._api, content[0], elasticube)

        return result

    def update(self) -> Resource:
        """Update the current data security rule.

        :return: (DataSecurity) updated.
        """
        self.fix_shares()
        data = self._payload(**self.json)

        content = self._api.put(f'elasticubes/datasecurity/{self._id}', data=data)

        return DataSecurity(self._api, content, self._elasticube)

    def delete(self):
        """Delete the current data security rule."""
        self._api.delete(f'elasticubes/datasecurity/{self._id}')

    def delete_all(self, table: str = None, column: str = None, elasticube: str = None):
        """
        Delete data security rules for a specific column.

        :param table: (str, default None) Datatable's name. If None, use self.table.
        :param column: (str, default None) Column's name. If None, use self.column.
        :param elasticube: (str, default None) Elasticube's name. If None, use self.elasticube.
        """
        elasticube = elasticube if elasticube else self._elasticube
        table = table if table else self.table
        column = column if column else self.column

        self._api.delete(f'elasticubes/localhost/{elasticube}/datasecurity', query={'table': table, 'column': column})

    def _payload(self,
                 table: str,
                 column: str,
                 datatype: str,
                 shares: list,
                 **kwargs):
        data = {
            'table': table,
            'column': column,
            'datatype': datatype,
            'shares': shares,
        }

        if 'exclusionary' in kwargs:
            data['exclusionary'] = kwargs['exclusionary']

        if 'members' in kwargs:
            data['members'] = kwargs['members']
            data['allMembers'] = None
        else:
            data['allMembers'] = kwargs['all_members']
            data['members'] = []

        return data



from .build import Build
from .datamodel import Datamodel
from .permission import Permission
from .hierarchy import Hierarchy
from .datasecurity import DataSecurity
from .connection import Connection
from .elasticube import Elasticube
from .jaql import JAQL



from sisense.resource import Resource


class User(Resource):

    def get(self, email: str = None, oid: str = None) -> Resource:
        """
        Get user by e-mail. At least one parameter should be set.

        :param email: (str, default None) User's e-mail.
        :param oid: (str, default None) User's ID.
        :return: User, if found. None, otherwise.
        """
        if oid:
            content = self._api.get(f'users/{oid}')
        elif email:
            content = self._api.get('users', query={'email': email})
            content = content[0] if len(content) else None
        else:
            raise ValueError('At least one parameter should be set.')

        return User(self._api, content) if content else None

    def all(self, **kwargs) -> list:
        """
        Get all users.

        :param kwargs: Positional optional arguments used to filter users:
            - userName: (str) Username.
            - email: (str) User's email.
            - firstName: (str) User's first name.
            - lastName: (str) User's last name.
            - roleId: (str) Role ID.
            - groupId: (str) Group ID.
            - active: (bool) Whether to get active users (True) or not (False).
            - origin: ('ad', 'sisense') User's origin.
            - ids: (list) Users' IDs.
            - fields: (list) Which fields to return.
            - sort: (str) Field in which the results should be sorted.
            - skip: (int) Number of results to skip from start.
            - limit: (int) Number of results returned.
            - expand: (list) Fields that should be expanded (substitutes their IDs with actual objects).
        :return: (list)
        """
        content = self._api.get('users', query=kwargs)
        content = [User(self._api, rjson) for rjson in content]

        return content

    def create(self, email: str, **kwargs) -> Resource:
        """
        Create a new user.

        :param email: (str) User's email.
        :param kwargs: (dict) Optional keyword arguments:
            - userName: (str) Username.
            - firstName: (str) First name.
            - lastName: (str) Last name.
            - roleId: (str) Role's ID.
            - groups: (list) Group's ID in which user is part of.
        :return: (User) Newly created user.
        """
        data = {'email': email}
        data.update(kwargs)

        content = self._api.post('users', data=data)
        return User(self._api, content)

    def update(self, **kwargs):
        """
        Update the current user.

        At least one argument must be set.

        :param kwargs: Positional optional arguments:
            - email: (str),
            - userName: (str) Username.
            - firstName: (str) First name.
            - lastName: (str) Last name.
            - roleId: (str) Role's ID.
            - groups: (list) Group's ID in which user is part of.
            - preferences: (dict) User's preferences such as 'localeId'.
            - uiSettings: (dict) User's UI settings.
        """
        content = self._api.patch(f'users/{self._id}', data=kwargs)
        self.json = content

    def delete(self):
        """Delete the current user."""
        self._api.delete(f'users/{self._id}')



from .test_case import TestCaseV1 as TestCase
from sisense.data import Connection
import unittest


class ConnectionTestCase(TestCase):

    def test_all(self):
        connections = self.sisense.connection.all(limit=10)

        self.assertEqual(len(connections), 10)

        for conn in connections:
            self.assertIsInstance(conn, Connection)

    def test_get(self):
        connection = self.sisense.connection.all(limit=1)[0]
        other_conn = self.sisense.connection.get(connection._id)

        self.assertEqual(connection._id, other_conn._id)

    def test_update(self):
        parameters = self.conn.parameters
        parameters['IsLive'] = 'false'

        updated_conn = self.conn.update(parameters=parameters)

        self.assertNotEqual(self.conn.parameters['IsLive'], updated_conn.parameters['IsLive'])

    def create(self):
        self.conn = self.sisense.connection.create(provider='Excel', parameters={'filesToImport': 'No existing file.txt'})

        self.assertEqual(self.conn.provider, 'Excel')
        self.assertTrue('filesToImport' in self.conn.parameters)

    def delete(self):
        if hasattr(self, 'conn'):
            self.conn.delete()


if __name__ == '__main__':
    unittest.main()



from .test_case import TestCaseV09 as TestCase
from sisense.data import Hierarchy
import unittest
import json


class HierarchyTestCase(TestCase):

    def test_all(self):
        results = self.sisense.hierarchy.all(elasticube=self.config['elasticube'])
        ids = [h._id for h in self.hierarchies]

        self.assertIsInstance(results, list)
        self.assertEqual(len(results), len(self.hierarchies))

        for h in results:
            self.assertIsInstance(h, Hierarchy)
            self.assertIn(h._id, ids)

    def test_get(self):
        h = self.sisense.hierarchy.get(self.hierarchies[0]._id, self.config['elasticube'])

        self.assertIsInstance(h, Hierarchy)
        self.assertEqual(h._id, self.hierarchies[0]._id)

    def create(self):
        hierarchy = self.sisense.hierarchy

        with open('tests/support_files/hierarchy.json', 'r') as file:
            hierarchies = json.load(file)

        for h in hierarchies:
            hierarchy.create(h['title'], h['levels'], h['alwaysIncluded'], self.config['elasticube'])

        self.hierarchies = hierarchy.all(self.config['elasticube'])
        self.assertEqual(len(self.hierarchies), len(hierarchies))

    def delete(self):
        hierarchy = self.sisense.hierarchy
        hierarchies = hierarchy.all(self.config['elasticube'])
        [hierarchy.delete() for hierarchy in hierarchies]

        results = hierarchy.all(self.config['elasticube'])
        self.assertEqual(len(results), 0)


if __name__ == '__main__':
    unittest.main()



from .test_case import TestCaseV2 as TestCase
import unittest
import requests
import json
import os

# TODO: change unit tests
class DatamodelTestCase(TestCase):
    pass
    # def setUp(self):
    #     super(DatamodelTestCase, self).setUp()
    #     self.datamodel = self.sisense.datamodel.all(oid=self.config['datamodel.oid'])
    #
    # def test_get_by_id(self):
    #     self.assertEqual(self.datamodel.title, self.config['datamodel'])
    #
    # def test_get_by_name(self):
    #     datamodel = self.datamodel.all(title=self.config['datamodel'])
    #     self.assertEqual(datamodel.oid, self.config['datamodel.oid'])
    #
    # def test_create(self):
    #     title = 'SisenseAPI (New) Example'
    #     new_datamodel = self.datamodel.create(title)
    #     server_datamodel = self.datamodel.all(title=title)
    #
    #     self.assertEqual(new_datamodel.oid, server_datamodel.oid)
    #     server_datamodel.delete()
    #
    # def test_clone(self):
    #     title = 'SisenseAPI (Clone) Example'
    #     new_datamodel = self.datamodel.clone(title)
    #     server_datamodel = self.datamodel.all(title=title)
    #
    #     self.assertEqual(new_datamodel.oid, server_datamodel.oid)
    #     server_datamodel.delete()
    #
    # def test_delete(self):
    #     new_datamodel = self.datamodel.create('SisenseAPI (Delete) Example')
    #     oid = new_datamodel.oid
    #     new_datamodel.delete()
    #
    #     try:
    #         server_datamodel = self.datamodel.all(oid=oid)
    #         self.assertIsNone(server_datamodel.oid)
    #     except requests.exceptions.HTTPError as error:
    #         self.assertEqual(error.response.status_code, 404)
    #
    # def test_export(self):
    #     with self.subTest(full=False):
    #         filename = 'datamodel.smodel'
    #         self.datamodel.export_datamodel(filename, full=False)
    #
    #         with open(filename, 'r') as file:
    #             schema = json.load(file)
    #
    #         self.assertEqual(schema['title'], self.datamodel.title)
    #
    #         os.system(f'rm {filename}')
    #
    #     with self.subTest(full=True):
    #         filename = 'datamodel.sdata'
    #         self.datamodel.export_datamodel(filename, full=True)
    #         new_datamodel = self.datamodel.import_datamodel('SisenseAPI (Import) Example', filename, full=True)
    #
    #         self.assertEqual(type(new_datamodel.oid), str)
    #         new_datamodel.delete()
    #
    #         os.system(f'rm {filename}')
    #
    # def test_import(self):
    #     title = 'SisenseAPI (Import) Example'
    #
    #     with self.subTest(full=False):
    #         filename = 'datamodel.smodel'
    #         self.datamodel.export_datamodel(filename, full=False)
    #         self.datamodel.import_datamodel(title, filename, full=False)
    #
    #         new_datamodel = self.datamodel.all(title=title)
    #         self.assertEqual(new_datamodel.title, title)
    #         self.assertNotEqual(new_datamodel.oid, self.datamodel.oid)
    #
    #         new_datamodel.delete()
    #         os.system(f'rm {filename}')
    #
    #     with self.subTest(full=True):
    #         filename = 'datamodel.sdata'
    #         self.datamodel.export_datamodel(filename, full=True)
    #         new_datamodel = self.datamodel.import_datamodel(title, filename, full=True)
    #
    #         self.assertEqual(new_datamodel.title, title)
    #         self.assertNotEqual(new_datamodel.oid, self.datamodel.oid)
    #
    #         new_datamodel.delete()
    #         os.system(f'rm {filename}')


if __name__ == '__main__':
    unittest.main()
